//main on SPI master
#include <zephyr.h>
#include "nrf.h"
#include <nrfx_spim.h>
#include <nrfx_gpiote.h>
#include <drivers/nrfx_errors.h>
#include <sys/printk.h>
#include <string.h>

#define SPI_INSTANCE 1
static const nrfx_spim_t spim = NRFX_SPIM_INSTANCE(SPI_INSTANCE);

#define APP_SPIM_CS_PIN (10)
#define APP_SPIM_SCK_PIN (11)
#define APP_SPIM_MISO_PIN (12)
#define APP_SPIM_MOSI_PIN (13)

#define NRFX_CUSTOM_ERROR_CODES 0 //used in nrfx_errors.h

static nrfx_spim_config_t spim_config =
	NRFX_SPIM_DEFAULT_CONFIG(APP_SPIM_SCK_PIN, APP_SPIM_MOSI_PIN, APP_SPIM_MISO_PIN, APP_SPIM_CS_PIN);

#define TEST_STRING "1234567890"
static uint8_t m_tx_buf[] = TEST_STRING;
static uint8_t m_rx_buf[sizeof(TEST_STRING) + 1];
static const uint8_t m_length = sizeof(m_tx_buf);

static volatile bool
	spim_xfer_done; /**< Flag used to indicate that SPIM instance completed the transfer. */

static void manual_isr_setup()
{
#ifdef NRF9160_XXAA
	IRQ_DIRECT_CONNECT(UARTE1_SPIM1_SPIS1_TWIM1_TWIS1_IRQn, 0,
			   nrfx_spim_1_irq_handler, 0);
	irq_enable(UARTE1_SPIM1_SPIS1_TWIM1_TWIS1_IRQn);

#else	
	IRQ_DIRECT_CONNECT(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn, 0,
			   nrfx_spim_1_irq_handler, 0);
	irq_enable(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn);
#endif
}

void spim_event_handler(nrfx_spim_evt_t const *p_event, void *p_context)
{
	printk("IRQ: %d\n", p_event->type);
	if (p_event->type == NRFX_SPIM_EVENT_DONE) {
		spim_xfer_done = true;
		printk("Transfer completed\n");
	}
    if (m_rx_buf[0] != 0)
    {
        printk("Received: ");
        for(int i = 0; i < sizeof(m_rx_buf); i++) {
			printk("%x ", m_rx_buf[i]);
		}
		printk("\n");
    }	
}

void main(void)
{
	printk("SPIM example\n");
	nrfx_spim_xfer_desc_t xfer_desc = NRFX_SPIM_XFER_TRX(m_tx_buf, m_length, m_rx_buf, m_length);

	if (NRFX_SUCCESS !=
	    nrfx_spim_init(&spim, &spim_config, spim_event_handler, NULL)) {
		printk("Init Failed\n");
		return;
	}

	manual_isr_setup();
	while(1) {
		memset(m_rx_buf, 0, m_length);
		spim_xfer_done = false;

		nrfx_err_t err_code = nrfx_spim_xfer(&spim, &xfer_desc, 0);

		if (err_code == NRFX_ERROR_BUSY) {
			printk("SPI busy\n");
		}

		else if (err_code != NRFX_SUCCESS){
			printk("Error code = %d\n", err_code);
		}

		while (!spim_xfer_done) {
			__WFE();
		}
		m_tx_buf[0]++;
		if (m_tx_buf[0] > '9') {
			m_tx_buf[0] = '0';
		}
		k_sleep(K_MSEC(1000));
	}
}